@@ -368,14 +368,14 @@ class BBCooker:
                 rdepends = self.status.rundeps[fn]
                 for package in rdepends:
                     depend_tree["rdepends-pkg"][package] = []
-                    for rdepend in rdepends[package]:
+                    for rdepend in bb.utils.explode_deps(rdepends[package]):
                         depend_tree["rdepends-pkg"][package].append(rdepend)
                     packages.append(package)
 
                 rrecs = self.status.runrecs[fn]
                 for package in rrecs:
                     depend_tree["rrecs-pkg"][package] = []
-                    for rdepend in rrecs[package]:
+                    for rdepend in bb.utils.explode_deps(rrecs[package]):
                         depend_tree["rrecs-pkg"][package].append(rdepend)
                     if not package in packages:
                         packages.append(package)
@@ -981,9 +981,9 @@ class CookerParser:
         self.pointer = 0
 
     def parse_next(self):
+        cooker = self.cooker
         if self.pointer < len(self.filelist):
             f = self.filelist[self.pointer]
-            cooker = self.cooker
 
             try:
                 fromCache, skipped, virtuals = cooker.bb_cache.loadData(f, cooker.configuration.data, cooker.status)
